import time

import numpy as np
from airobot import Robot
from airobot.utils.common import euler2quat
import pybullet as p
import cv2
import math

# Code partially borrowed from https://gis.stackexchange.com/questions/76745/creating-a-circle-with-points

def rotate(x, y, r):
	"""
	Two dimensional rotation
	returns coordinates in a tuple (x,y)
	"""
	rx = (x*math.cos(r)) - (y*math.sin(r))
	ry = (y*math.cos(r)) + (x*math.sin(r))
	return (rx, ry)

def point_ring(center, num_points, radius):
	"""
	create a ring of points centered on center (x,y) with a given radius
	using the specified number of points
	center should be a tuple or list of coordinates (x,y)
	returns a list of point coordinates in tuples
	"""
	arc = (2 * math.pi) / num_points 
	points = []
	for p in range(num_points):
		(px,py) = rotate(0, radius, arc * p) 
		px += center[0]
		py += center[1]
		# Assumes height is fixed 
		points.append((px,py,1.0))
	return points

def main():
	# Creating the robot
	robot = Robot('ur5e_stick', pb=True, pb_cfg={'gui': False})

	# Takes the robot arm to default position
	robot.arm.go_home(ignore_physics=True)

	# Let's add a camera to the robot for getting images for visualization 
	# You can get images by setting 'gui' to True above. But if you are running
	# your file on some server, it will be harder to implement this option.
	robot_base = robot.arm.robot_base_pos
	robot.cam.setup_camera(focus_pt=robot_base, dist=3, yaw=55, pitch=-30, roll=0)

	# We will save the images in form of video
	out = cv2.VideoWriter('hw0.mp4', cv2.VideoWriter_fourcc(*'mp4v'), 30, (640,480)) 

	# Add a table
	table_id = robot.pb_client.load_urdf('table/table.urdf',[.5, 0, 0.4], euler2quat([0, 0, np.pi / 2]), scaling=0.9)

	# We will move the robot's tip in a circle

	# First collect the points on the circle
	points = point_ring(center=(0.5,0), radius=0.2, num_points=200)

	# Let's move the robot along the points on circle
	img = robot.cam.get_images(get_rgb=True, get_depth=False)[0]
	out.write(np.array(img))

	for pt in points:
		robot.arm.set_ee_pose(pt)
		img = robot.cam.get_images(get_rgb=True, get_depth=False)[0]
		out.write(np.array(img))

	out.release()


if __name__ == '__main__':
	main()
